/**
 * This file is part of pppBenchmark, a program accompanying
 * the paper: Abeel, T, Van de Peer Y, Saeys Y.  2009.  Towards a gold standard for promoter prediction evaluation. Bioinformatics. 25(12):i313-i320.
    * 
 * Copyright 2009 Thomas Abeel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *
 */
package be.abeel.pppbenchmark;

import java.io.File;
import java.text.NumberFormat;
import java.util.BitSet;
import java.util.HashMap;
import java.util.Map;

import be.abeel.io.LineIterator;
import be.abeel.io.LineIteratorFactory;

/**
 * protocol based on the work by Sonnenburg et al.
 * 
 * @author Thomas Abeel
 * 
 */
public class BinTagsProtocol extends Protocol{

    public Map<Double, int[]> run(String chr, LineIterator pred) throws Exception {
    	System.out.println("\tRunning BinTags: "+chr);
    	Map<Double, int[]>out=new HashMap<Double, int[]>();
        BitSet tags = new BitSet((int) (Config.chrSize.get(chr) / Config.distance) + 1);
        double[] scores = new double[(int) (Config.chrSize.get(chr) / Config.distance) + 1];
        int[]indices=new int[(int) (Config.chrSize.get(chr) / Config.distance) + 1];
//        double min = minScore.get(Config.predName(pred));
//        double max = maxScore.get(Config.predName(pred));

        for (int i = 0; i < scores.length; i++) {
            scores[i] = Double.NEGATIVE_INFINITY;
        }
        /* Load tags for the selected chromosome */
        LineIterator it = new LineIterator(Config.cage());
        for (String line : it) {
            String[] arr = line.split("\t");
            if (arr[0].equalsIgnoreCase(chr)) {
                int start = Integer.parseInt(arr[3]);
                int end = Integer.parseInt(arr[4]);
                int middle=start/2+end/2;
//                for (int i = (start - 20) / chunksize; i <= (end + 20) / chunksize; i++) {
                    // System.out.println("Start=" + start + "\tend=" + end +
                    // "\tset=" + i);
                    tags.set(middle/Config.distance);
//                }
            }

        }
        it.close();
        /* Load predictions for the selected chromosome */
        
        for (String line : pred) {
            // System.err.println(line);
            String[] arr = line.split("\t");
            if (arr[0].equalsIgnoreCase(chr)) {
                int start = Integer.parseInt(arr[1]);
                int end = Integer.parseInt(arr[2]);
                int middle=start/2+end/2;
                double score = Double.parseDouble(arr[3]);
                // System.out.println(score);
//                if (score > max)
//                    max = score;
//                if (score < min)
//                    min = score;
//
//                for (int i = (middel - 20) / chunksize; i <= (end + 20) / chunksize; i++) {
                    
                    // System.out.println("Start=" + start + "\tend=" + end +
                    // "\tset=" + i);
                    if (scores[middle/Config.distance] < score)
                        scores[middle/Config.distance] = score;
//                }
            }

        }
        pred.close();
//        System.out.println("# chromosome="+chr);
//        System.out.println("# Distinct tags set = " + tags.cardinality());
//        System.out.println("# Sites = " + scores.length);
     
        /* Do analysis with moving threshold */
        double range = 1 / Config.steps;
        if(range<0.000001){
            range=1000;
            System.err.println("BinTags: Modified range!");
        }
//        System.out.println("# thres\t#pred\tPM\tprec\trecall");
        for (double d = 0; d <= 1; d += range) {
            BitSet prediction = new BitSet((int) (Config.chrSize.get(chr) / Config.distance) + 1);
            for (int i = 0; i < scores.length; i++) {
                if (scores[i] >= d)
                    prediction.set(i);
            }
            // System.out.println("threshold d=" + d);
            // System.out.println("Distinct predictions = " +
            // prediction.cardinality());
            int tp = 0, tn = 0, fp = 0, fn = 0;
            for (int i = 0; i < scores.length; i++) {
                if (tags.get(i) && prediction.get(i)) {
                    tp++;
                }
                if (tags.get(i) && !prediction.get(i)) {
                    fn++;
                }
                if (!tags.get(i) && prediction.get(i)) {
                    fp++;
                }
                if (!tags.get(i) && !prediction.get(i)) {
                    tn++;
                }
            }
//            PerformanceMeasure pm = new PerformanceMeasure(tp, tn, fp, fn);
            // System.out.println("BCR=" + pm.getBCR());
            // System.out.println("F=" + pm.getFMeasure());
            NumberFormat nf = NumberFormat.getInstance();
            nf.setMaximumFractionDigits(2);
//            System.out.println(d + "\t" + prediction.cardinality()  + "\t"
//                    + pm + "\t" + nf.format(pm.getPrecision()) + "\t"
//                    + nf.format(pm.getRecall()));
//            System.out.println(d + "\t" + prediction.cardinality()  + "\t" + nf.format(getPrecision(tp,tn,fp,fn))
//                    + "\t" + nf.format(getRecall(tp, tn, fp, fn)));
            // System.out.println("TP=" + tp);
            // System.out.println("FP=" + fp);
            // System.out.println("TN=" + tn);
            // System.out.println("FN=" + fn);
            out.put(d, new int[]{tp,tn,fp,fn});
            

        }
        return out;

    }
   
}
