/**
 * This file is part of pppBenchmark, a program accompanying
 * the paper: Abeel, T, Van de Peer Y, Saeys Y.  2009.  Towards a gold standard for promoter prediction evaluation. Bioinformatics. 25(12):i313-i320.
    * 
 * Copyright 2009 Thomas Abeel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *
 */
package be.abeel.pppbenchmark;

import java.io.File;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import be.abeel.io.LineIterator;
import be.abeel.io.LineIteratorFactory;

/**
 * Protocol based on the work by Abeel et al. 2008
 * 
 * @author Thomas Abeel
 * 
 */
public class PredGeneProtocol extends Protocol{

   
    public Map<Double, int[]> run(String chr,LineIterator pred) throws Exception {
    	System.out.println("\tRunning PredGene: "+chr);
    	   Map<Double,int[]>out=new HashMap<Double, int[]>();
//        System.out.println("# chromosome=" + chr);
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(5);

        ArrayList<Gene> genes = new ArrayList<Gene>();
        ArrayList<Prediction> predictions = new ArrayList<Prediction>();

        /* Load genes for the selected chromosome */
        LineIterator it = new LineIterator(Config.gene());
        for (String line : it) {
            String[] arr = line.split("\t");
            if (arr[0].equalsIgnoreCase(chr)) {
                int start = Integer.parseInt(arr[1]);
                int end = Integer.parseInt(arr[2]);
                boolean forward = arr[5].equals("+");
                genes.add(new Gene(start, end, forward));
            }

        }
        it.close();
        /* Load predictions for the selected chromosome */
        
        for (String line : pred) {
            // System.err.println(line);
            String[] arr = line.split("\t");
            if (arr[0].equalsIgnoreCase(chr)) {
                int start = Integer.parseInt(arr[1]);
                int end = Integer.parseInt(arr[2]);
                double score = Double.parseDouble(arr[3]);
                predictions.add(new Prediction(start, end, score));

            }

        }
        pred.close();

        
//        System.out.println("# Distinct genes set = " + genes.size());
//        System.out.println("# Predictions = " + predictions.size());
//      
        /* Do analysis with moving threshold */
        for (Gene g : genes) {
            for (Prediction p : predictions) {
                if (g.overlaps(p)) {

                    p.overlap = true;

                }
                if (g.nearStart(p, Config.distance)) {
                    p.nearGeneStart = true;
                    g.setLowest(p.score);
                }
            }
        }

        double range = (1) / Config.steps;
        if (range == 0)
            range++;
//        System.out.println("#\tpred (corr/tot)\ttags (corr/tot)\tprec.\trecall");
        for (double d = 0; d <= 1; d += range) {
            int predictedTags = 0;
            int totalTags = 0;
            int correctPredictions = 0;
            int totalPredictions = 0;
            for (Gene g : genes) {
                totalTags++;
                if (g.highestScore >= d)
                    predictedTags++;
            }
            for (Prediction p : predictions) {
                if (p.score >= d && p.nearGeneStart) {
                    correctPredictions++;
                    totalPredictions++;
                } else if (p.score >= d && p.overlap) {
                    totalPredictions++;
                }
            }
//            System.out.println(nf.format(d) + "\t" + correctPredictions + "\t" + totalPredictions + "\t"
//                    + predictedTags + "\t" + totalTags + "\t"
//                    + nf.format((double) correctPredictions / totalPredictions) + "\t"
//                    + nf.format((double) predictedTags / totalTags));
            out.put(d, new int[]{correctPredictions,totalPredictions,predictedTags,totalTags});
        }
        return out;

    }
}
